var buddyTree = {
	data : new Array(),
    rowCount : 0,
    getCellText : function(row,column) {		// names for the directories
		if (row == -1)
			return "";
		if (this.data[row].type == "Buddy")
			return this.data[row].name;
		var online = 0;
		var total;
		if (this.data[row].children.length)
			online = this.data[row].children.length;
		else {
			var x;
			for (x = row + 1; x < this.data.length; ++x)
				if (this.data[x].type != "Buddy")
					break;
			online = x - 1 - row;
		}
		total = online;
		if (this.data[row].offline != null)
			total += this.data[row].offline.length;
		return this.data[row].name + "  (" + online + "/" + total + ")";
    },
	getImageSrc : function(row,col) {			// icons for the directories
		if (this.data[row].type == "Group")
			return null;
		if (this.data[row].permit == "F")
			return "res/denied.png";
		if (this.data[row].userClass.indexOf('U') == 2)
			return "res/away.ico";
		if (this.data[row].idleTime > 5)
			return "res/idle.png";
		if (this.data[row].online)
			return "res/active.gif";
		else
			return null;
	},
	getLevel : function(row) {
		if (row <= 0 || row >= this.data.length || this.data[row].type == "Group")
			return 0;
		return 1;
	},
	getParentIndex : function(row) {
		if (row <= 0 || this.data[row].type == "Group")
			return -1;
		for (var x = row - 1; x >= 0; --x)
			if (this.data[x].type == "Group")
				return x;
		return -1;									// shouldn't get here
	},
	getCellProperties	: function(row,col,props)	{},
    getColumnProperties	: function(colid,col,props)	{},
	getRowProperties	: function(row,props)		{},
 	hasNextSibling		: function(row,nextrow)		{
		if (this.data[row].type == "Buddy")
			return row != this.rowCount - 1 && this.data[row + 1].type != "Group";
		for (var x = row + 1; row < this.rowCount; ++x)
			if (this.data[x].type == "Group")
				return true;
		return false;
	},
  	isContainer			: function(row)				{ return this.data[row].type == "Group" },
 	isContainerEmpty	: function(row)				{
		return this.data[row].type == "Group" && !this.data[row].children.length && (row == this.rowCount - 1 || this.data[row + 1].type == "Group");
	},
	isContainerOpen		: function(row)				{ return this.data[row].type == "Group" && !this.isContainerEmpty(row) && this.data[row].open; },
	isSeparator			: function(row)				{ return false; },
    isSorted			: function(row)				{ return false; },
    setTree				: function(treebox)			{ this.treebox = treebox; },
	toggleOpenState		: function(row)				{
		if (this.isContainerOpen(row)) {
			var level = this.getLevel(row);
			var lastChild = row;
			while (lastChild + 1 < this.rowCount && this.getLevel(lastChild + 1) > level)		// find last index in same level as collapsed dir	
				++lastChild;
			this.data[row].children = this.data.splice(row + 1, lastChild - row);				// get rid of subdirectories from view
			this.rowCount = this.data.length;
			this.treebox.rowCountChanged(row, -(lastChild - row));

			this.data[row].open = false;
			this.treebox.invalidateRow(row);													// update row
		} else {
			for (var x = this.data[row].children.length - 1; x >= 0; --x)
				this.data.splice(row + 1, 0, this.data[row].children[x]);
			this.rowCount = this.data.length;
			this.treebox.rowCountChanged(row + 1, this.data[row].children.length);
			this.data[row].children = new Array();
			this.data[row].open = true;
			this.treebox.invalidateRow(row);
		}
	},
	IM : function(event) {
		if (event && event.button != 0)
			return;
		var username;
		if (!event) {
			username = window.prompt("Please type in the name of the user you'd like to talk to:", "", "IM");
			if (!username)
				return null;
		} else {
			if (this.data[this.selection.currentIndex].type == "Group")
				return;
			username = this.data[this.selection.currentIndex].name;
		}
		aimfire.im(mozillaTOC, username, mozillaTOC.id, null, null);
	},
	getInfo : function() {
		var username;
		if (this.selection.currentIndex < 0 || this.selection.currentIndex >= this.rowCount || this.data[this.selection.currentIndex].type == "Group") {
			username = window.prompt("Please type in the name of the user you'd like to get info on:", "", "Get Info");
			if (!username)
				return null;
		} else
			username = this.data[this.selection.currentIndex].name;
		mozillaTOC.getInfo(username);
	},
	setInfo : function() {
		var newWindow = window.openDialog("chrome://aimfire/content/infoWindow.xul","infoWindow",
				"screenY="+(document.getElementById('main-window').boxObject.height/2 - 45+document.getElementById('main-window').boxObject.screenY)
			  +",screenX="+(document.getElementById('main-window').boxObject.width/2 - 200+document.getElementById('main-window').boxObject.screenX)+
				",width=450,height=400,chrome,modal=no,dialog=yes,resizable=yes", mozillaTOC, audioscrobblerID);
	},
	updateBuddyList : function(buddy, online, signonTime, idleTime, userClass) {
		var newBuddyInfo = { name: buddy, type: "Buddy", online: online, signonTime: signonTime, idleTime: idleTime, userClass: userClass, permit: "" };
		for (var x = 0; x < this.data.length; ++x)
			if (this.data[x].type == "Buddy" && mozillaTOC.normalize(this.data[x].name) == mozillaTOC.normalize(buddy)) {
				newBuddyInfo.permit = this.data[x].permit;
				if (newBuddyInfo.online) {
					this.data[x] = newBuddyInfo;
					this.treebox.invalidateRow(x);
				} else {
					var parentIndex = this.getParentIndex(x);
					this.data[parentIndex].offline.push(newBuddyInfo);
					this.data.splice(x, 1);
					this.rowCount = this.data.length;
					this.treebox.rowCountChanged(x, -1);
					this.treebox.invalidateRow(parentIndex);
				}
			} else if (this.data[x].type == "Group" && this.data[x].children.length) {	// group is collapsed
				for (var y = 0; y < this.data[x].children.length; ++y)
					if (mozillaTOC.normalize(this.data[x].children[y].name) == mozillaTOC.normalize(buddy)) {
						newBuddyInfo.permit = this.data[x].children[y].permit;
						if (newBuddyInfo.online)
							this.data[x].children[y] = newBuddyInfo;
						else {
							this.data[x].offline.push(newBuddyInfo);
							this.data[x].children.splice(y, 1);
						}
					}
			} else if (this.data[x].type == "Group" && this.data[x].offline.length) {	// group isn't collapsed
				for (var y = 0; y < this.data[x].offline.length; ++y)
					if (mozillaTOC.normalize(this.data[x].offline[y].name) == mozillaTOC.normalize(buddy)) {
						newBuddyInfo.permit = this.data[x].offline[y].permit;
						if (newBuddyInfo.online) {
							this.data[x].offline.splice(y, 1);
							var index;
							for (index = x + 1; index < this.data.length; ++index)
								if (this.data[index].type == "Group")
									break;
							var children = this.data.splice(x + 1, index - (x + 1));
							children.push(newBuddyInfo);
							children.sort(this.buddySort);
							for (var z = children.length - 1; z >= 0; --z)
								this.data.splice(x + 1, 0, children[z]);
							//this.data.splice(index, 0, newBuddyInfo);
							
							this.rowCount = this.data.length;
							this.treebox.rowCountChanged(index - 1, 1);
							this.treebox.invalidateRange(x, index - 1);
						} else {
							this.data[x].offline[y] = newBuddyInfo;
						}
					}
			}
		
		//var tooltipSignonTime = months[signonTime.getUTCMonth()] + " " + signonTime.getDate() + " " +  signonTime.toLocaleTimeString();
		//var tooltip = online ? ("Online since: " + tooltipSignonTime + ", Idle: " + idleTime + " minute(s)") : "Offline";
	},
	addBuddy : function() {
		username = window.prompt("Username:", "", "Add Buddy");
		if (!username)
			return;
		mozillaTOC.addBuddy(username);	// do this after?
		var index = this.selection.currentIndex < 0 || this.selection.currentIndex >= this.rowCount ? this.data.length - 1 : this.selection.currentIndex;
		var newBuddy = { name: username, type: "Buddy", online: false, signonTime: null, idleTime: 0, userClass: "", permit: "" };
		if (this.data[index].type == "Group")
			this.data[index].offline.push(newBuddy);
		else
			this.data[this.getParentIndex(index)].offline.push(newBuddy);
		this.setConfig();
	},
	addGroup : function() {
		group = window.prompt("Group:", "", "Add Group");
		if (!group)
			return;
		this.data.push({ name: group, type: "Group", open: true, children: new Array(), offline: new Array() });
		this.rowCount = this.data.length;
		this.treebox.rowCountChanged(this.rowCount - 1, 1);
		this.setConfig();
	},
	remove : function() {
		if (this.selection.currentIndex < 0 || this.selection.currentIndex >= this.rowCount)
			return;
		var index = this.selection.currentIndex;
		if (this.data[index].type == "Buddy") {
			mozillaTOC.removeBuddy(this.data[index].name);
			var parentIndex = this.getParentIndex(index);
			this.data.splice(index, 1);
			this.rowCount = this.data.length;
			this.treebox.rowCountChanged(index, -1);
			this.treebox.invalidateRow(parentIndex);
		} else {
			if (!confirm("Are you sure you want to delete the entire group?"))
				return;
			if (this.data[index].offline != null)
				for (var x = 0; x < this.data[index].offline.length; ++x)
					mozillaTOC.removeBuddy(this.data[index].offline[x].name);
			if (this.data[index].children.length) {
				for (var x = 0; x < this.data[index].children.length; ++x)
					mozillaTOC.removeBuddy(this.data[x].name);
				this.data.splice(index, 1);
				this.rowCount = this.data.length;
				this.treebox.rowCountChanged(index, -1);
			} else {
				var x;
				for (x = index + 1; x < this.data.length && this.data[x].type != "Group"; ++x)
					mozillaTOC.removeBuddy(this.data[x].name);
				this.data.splice(index, x - index);
				this.rowCount = this.data.length;
				this.treebox.rowCountChanged(index, -(x - index));
			}
		}
		if (!this.data.length) {	// empty config - add at least one group
			this.data.push({ name: "Buddies", type: "Group", open: true, children: new Array(), offline: new Array() });
			this.rowCount = 1;
			this.treebox.rowCountChanged(0, 1);
		}
		this.setConfig();
	},
	removeOffline : function() {
		username = window.prompt("Username:", "", "Remove Buddy");
		if (!username)
			return;
		for (var x = 0; x < this.data.length; ++x)
			if (this.data[x].type == "Group") {
				for (var y = 0; y < this.data[x].offline.length; ++y)
					if (mozillaTOC.normalize(this.data[x].offline[y].name) == mozillaTOC.normalize(username)) {
						this.data[x].offline.splice(y, 1);
						this.treebox.invalidateRow(x);
						break;
					}
			}
	},
	permit : function() {
		if (this.selection.currentIndex < 0 || this.selection.currentIndex >= this.rowCount)
			return;
		this.data[this.selection.currentIndex].permit = "T";
		this.treebox.invalidateRow(this.selection.currentIndex);
		mozillaTOC.permitBuddy(this.data[this.selection.currentIndex].name);
		this.setConfig();
	},
	deny : function() {
		if (this.selection.currentIndex < 0 || this.selection.currentIndex >= this.rowCount)
			return;
		this.data[this.selection.currentIndex].permit = "F";
		this.treebox.invalidateRow(this.selection.currentIndex);
		mozillaTOC.denyBuddy(this.data[this.selection.currentIndex].name);
		this.setConfig();
	},
	setConfig : function() {
		var configData = "m 1\n";//xxx deal with other permissions
		for (var x = 0; x < this.data.length; ++x) {
			if (this.data[x].type == "Group") {
				configData += "g " + this.data[x].name + "\n";
				if (this.data[x].children.length) {
					for (var y = 0; y < this.data[x].children.length; ++y) {
						configData += "b " + mozillaTOC.normalize(this.data[x].children[y].name) + "\n";
						if (this.data[x].children[y].permit == "T")
							configData += "p " + mozillaTOC.normalize(this.data[x].children[y].name) + "\n";
						else if (this.data[x].children[y].permit == "F")
							configData += "d " + mozillaTOC.normalize(this.data[x].children[y].name) + "\n";
					}
				}
				if (this.data[x].offline != null) {
					for (var y = 0; y < this.data[x].offline.length; ++y) {
						configData += "b " + mozillaTOC.normalize(this.data[x].offline[y].name) + "\n";
						if (this.data[x].offline[y].permit == "T")
							configData += "p " + mozillaTOC.normalize(this.data[x].offline[y].name) + "\n";
						else if (this.data[x].offline[y].permit == "F")
							configData += "d " + mozillaTOC.normalize(this.data[x].offline[y].name) + "\n";
					}
				}
			} else {
				configData += "b " + mozillaTOC.normalize(this.data[x].name) + "\n";
				if (this.data[x].permit == "T")
					configData += "p " + mozillaTOC.normalize(this.data[x].name) + "\n";
				else if (this.data[x].permit == "F")
					configData += "d " + mozillaTOC.normalize(this.data[x].name) + "\n";
			}
		}
		mozillaTOC.setConfig(configData);
	},
	buddySort : function(a, b) {
		if (mozillaTOC.normalize(a.name) < mozillaTOC.normalize(b.name))
			return -1;
		if (mozillaTOC.normalize(a.name) > mozillaTOC.normalize(b.name))
			return 1;
		return 0;
	},
};
