/*
 * Translated into Javascript and slightly modified
 * by Mime Cuvalo, April 2005
 *
 * The JavaTOC framework is a set of classes used to allow
 * a Java program to communicate with AOL's TOC protocol.
 * The Chatable interface and JavaTOC classes can easily
 * be moved to any program needing TOC abilities.
 *
 *  Copyright 2002 by Jeff Heaton
 *
 *   This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * @author Jeff Heaton(http://www.jeffheaton.com)
 * @version 1.0
 * 
 */

var mozillaTOC = {
	active		: false,						// Is this service connected?
	tocHost		: "toc.oscar.aol.com",			// The host address of the TOC server
	tocPort		: 9898,							// The port used to connect to the TOC server
	authHost	: "login.oscar.aol.com",		// The OSCAR authentication server
	authPort	: 5190,							// The OSCAR authentication server's port
	language	: "english",					// What language to use
	version		: "AIMfire by Mime Cuvalo",		// The version of the client
	roastString	: "Tic/Toc",					// The string used to "roast" passwords
	sequence	: 0,							// The sequence number used for FLAP packets
	connection	: null,							// The connection to the TOC server
	is			: null,							// An InputStream to the connection
	os			: null,							// An OutputStream to the connection
	id			: "",							// Screen name of current user
	password	: "",							// Password of current user
	SIGNON		: 1,							// The ID number for a SIGNON packet(FLAP)
	DATA		: 2,							// The ID number for a DATA packet (flap)
	BUDDY_ICON	: "09461346-4C7F-11D1-8222-444553540000", 	// buddy icon uuid

	login : function(id, password) {
		try {
			document.getElementById('login').collapsed = true;
			document.getElementById('statusmeter').setAttribute("mode", "undetermined");
			document.getElementById('connecting').collapsed = false;
			this.id = id;
			this.password = password;

			// get IP we're connected to - toc.oscar.aol.com resolves to multiple ip addresses
			var DNSService = Components.classes["@mozilla.org/network/dns-service;1"].getService(Components.interfaces.nsIDNSService);
			this.tocHost = DNSService.resolve(this.tocHost, true).getNextAddrAsString();

			var proxyInfo = null;

			if (document.getElementById("proxy").checked) {													// use a proxy
				var prefs = Components.classes["@mozilla.org/preferences-service;1"].
				getService(Components.interfaces.nsIPrefService);
				var prefBranch = prefs.getBranch("network.proxy.");

				var proxyService = Components.classes["@mozilla.org/network/protocol-proxy-service;1"]
					.getService(Components.interfaces.nsIProtocolProxyService);
				proxyInfo = proxyService.newProxyInfo("socks" + prefBranch.getIntPref("socks_version"),
				prefBranch.getCharPref("socks"), prefBranch.getIntPref("socks_port"));
			}

			var transportService = Components.classes["@mozilla.org/network/socket-transport-service;1"]
				.getService(Components.interfaces.nsISocketTransportService);
			this.connection = transportService.createTransport(null, 0, this.tocHost, this.tocPort, proxyInfo);

			var inStream = this.connection.openInputStream(0,0,0);
			this.is = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(Components.interfaces.nsIBinaryInputStream);
			this.is.setInputStream(inStream);

			var outStream = this.connection.openOutputStream(0,0,0);
			this.os = Components.classes["@mozilla.org/binaryoutputstream;1"].createInstance(Components.interfaces.nsIBinaryOutputStream);
			this.os.setOutputStream(outStream);

			var dataListener = {																// async data listener for the control socket
				data : "",
				state : 2,
				onStartRequest: function(request, context) {},
				onStopRequest: function(request, context, status) { },
				onDataAvailable: function(request, context, inputStream, offset, count) {
					var str = mozillaTOC.getFlap();
					if (this.state == 0) {
						mozillaTOC.processTOCEvents(str);
					} else if (this.state == 1) {
						this.state = 0;
						mozillaTOC.login3(str);
					} else {
						this.state = 1;
						mozillaTOC.login2();
					}
				},
			};
			var pump = Components.classes["@mozilla.org/network/input-stream-pump;1"].
				createInstance(Components.interfaces.nsIInputStreamPump);
			pump.init(inStream, -1, -1, 0, 0, false);
			pump.asyncRead(dataListener, null);

			this.sendRaw("FLAPON\r\n\r\n");
		} catch (ex) {		// disconnected
			alert('Unable to make a connection.');
			this.logout();
		}
	},
	login2 : function() {
		this.sendFlapSignon();
		var command =	"toc_signon " +
						this.authHost + " " +
						this.authPort + " " +
						this.id + " " +
						this.roastPassword(this.password) + " " +
						this.language + " \"" +
						this.version + "\"";

		this.sendFlap(this.DATA, command);
	},
	login3 : function(str) {
 		document.getElementById('connecting').collapsed = true;
		document.getElementById('statusmeter').setAttribute("mode", "determined");
		if (str && str.toUpperCase().indexOf("ERROR:") == 0) {
			this.handleError(str);
			document.getElementById('login').collapsed = false;
			return;
		}

		this.sendFlap(this.DATA, "toc_add_buddy " + this.normalize(this.id));
		this.sendFlap(this.DATA, "toc_init_done");
		//this.sendFlap(this.DATA, "toc_set_caps ");
		this.sendFlap(this.DATA, "toc_add_permit ");
		this.sendFlap(this.DATA, "toc_add_deny ");
		if (audioscrobblerID)
			userInfo += "<br><a href='http://audioscrobbler.com/user/" + audioscrobblerID + "'>Audioscrobbler Profile</a>"
		if (userInfo)
			this.sendFlap(this.DATA, "toc_set_info " + "\"" + userInfo + "\"");
		this.active = true;

		document.getElementById('buddyView').collapsed = false;
		aimfire.savePassword();
	},
	logout : function() {
		try {
			document.getElementById('connecting').collapsed = true;
			document.getElementById('buddyView').collapsed = true;
			document.getElementById('statusmeter').setAttribute("mode", "determined");
			document.getElementById('login').collapsed = false;
			this.connection.close("");
			this.is.close();
			this.os.close();
			this.active = false;
		} catch (ex) { }
	},

   /**
	* Called to roast the password.
	*
	* Passwords are roasted when sent to the host.  This is done so they
	* aren't sent in "clear text" over the wire, although they are still
	* trivial to decode.  Roasting is performed by first xoring each byte
	* in the password with the equivalent modulo byte in the roasting
	* string.  The result is then converted to ascii hex, and prepended
	* with "0x".  So for example the password "password" roasts to
	* "0x2408105c23001130"
	*/
	roastPassword : function(str) {
		var xor = this.roastString;
		var xorIndex = 0;
		var rtn = "0x";

		for (var i = 0; i < str.length; i++) {
			var hex = (xor[xorIndex].charCodeAt(0) ^ str.charCodeAt(i)).toString(16);
			if (hex.length == 1)
				hex = "0" + hex;
			rtn += hex;
			xorIndex++;
			if (xorIndex == xor.length)
				xorIndex = 0;
		}
		return rtn;
	},
	sendRaw : function(str) {			// Send a string over the socket as raw bytes
		this.os.writeBytes(str, str.length);
	},
	writeWord : function(word) {		// Write a little endian word
		try {
			this.os.write8(((word >> 8) & 0xff));
			this.os.write8((word & 0xff));
		} catch (ex) {
			alert('Connection lost.');
			this.logout();
		}
	},
	sendFlapSignon : function() {		// Send a FLAP signon packet
		try {
			var length = 8 + this.id.length;
			this.sequence++;
			this.os.writeBytes('*', 1);
			this.os.write8(this.SIGNON);
			this.writeWord(this.sequence);
			this.writeWord(length);

			this.os.write8(0);
			this.os.write8(0);
			this.os.write8(0);
			this.os.write8(1);

			this.os.write8(0);
			this.os.write8(1);

			this.writeWord(this.id.length);
			this.os.writeBytes(this.id, this.id.length);
			this.os.flush();
		} catch (ex) {
			alert('Connection lost.');
			this.logout();
		}
	},
	sendFlap : function(type, str) {	// Send a FLAP packet
		try {
			if (str.length > 2048) {		// message is too large	
				alert(error);				// xxx add to im window
				return;
			}
			var length = str.length + 1;
			this.sequence++;
			this.os.writeBytes('*', 1);
			this.os.write8(type);
			this.writeWord(this.sequence);
			this.writeWord(length);
			this.os.writeBytes(str, str.length);
			this.os.write8(0);
			this.os.flush();
		} catch (ex) {
			alert('Connection lost.');
			this.logout();
		}
	},
	getFlap : function() {				// Get a FLAP packet
		try {
			if (this.is.read8() != '*'.charCodeAt(0))
				return null;
			this.is.read8();
			this.is.read8();
			this.is.read8();
			var length = (this.is.read8() * 0x100) + this.is.read8();
			var b = "";
			for (var x = 0; x < length; ++x)
				b += String.fromCharCode(this.is.read8());
			//this.is.readBytes(length, b);
			return b;
		} catch (ex) { }
	},
	processTOCEvents : function(str) {		// Process all TOC events
		if (str.toUpperCase().indexOf("IM_IN:") == 0)
			this.handleIM(str);
		else if (str.toUpperCase().indexOf("UPDATE_BUDDY:") == 0)
			this.updateBuddy(str);
		else if (str.toUpperCase().indexOf("CONFIG:") == 0)
			this.getConfig(str);
		else if (str.toUpperCase().indexOf("GOTO_URL:") == 0)
			this.displayInfo(str);
		else if (str.toUpperCase().indexOf("RVOUS_PROPOSE") == 0)
			this.handleRvous(str);
		else if (str.toUpperCase().indexOf("ERROR:") == 0)
			this.handleError(str);
	},
	handleIM : function(str) {			// Parse an instant message and send it back to the aimfire
		var tok = str.split(":");
		if (tok.length < 2)
			return;

		var from = tok[1];
		var autoResponse = tok[2] == "T";
		var message = tok[3];
		for (var x = 4; x < tok.length; ++x)
			message += tok[x] ? (":" + tok[x]) : "";

		aimfire.im(this, from, this.id, message, autoResponse);
	},
	updateBuddy : function(str) {
		var tok = str.split(":");
		buddyTree.updateBuddyList(tok[1], tok[2] == "T", new Date(parseInt(tok[4])), tok[5], tok[6]);
	},
	getConfig : function(str) {
		var names = (str.split(":"))[1].split("\n");
		names.splice(names.length - 1, 1);	// last item is invalid
		var buddies = new Array();
		
		var index = -1;
		for (var x = 0; x < names.length; ++x)
			if (names[x].charAt(0) == 'm')
				index = x;
		if (index != -1)		// XXX deal with config:m 1234
			names.splice(index, 1);

		if (!names.length || names[0].charAt(0) != "g") // empty config - add at least one group
			buddies.push({ name: "Buddies", type: "Group", open: true, children: new Array(), offline: new Array() });
		var lastParent = 0;
		for (var x = 0; x < names.length; ++x) {
			if (names[x].charAt(0) == 'g') {
				buddies.push({ name: names[x].substring(2, names[x].length), type: "Group", open: true, children: new Array(), offline: new Array() });
				lastParent = buddies.length - 1;
			}
			else if (names[x].charAt(0) == 'b')
				buddies[lastParent].offline.push({ name: names[x].substring(2, names[x].length), type: "Buddy", online: false, signonTime: null, idleTime: 0, userClass: "", permit: "" });
			else if (names[x].charAt(0) == 'p')
				buddies[lastParent].offline.push({ name: names[x].substring(2, names[x].length), type: "Buddy", online: false, signonTime: null, idleTime: 0, userClass: "", permit: "T" });
			else if (names[x].charAt(0) == 'd')
				buddies[lastParent].offline.push({ name: names[x].substring(2, names[x].length), type: "Buddy", online: false, signonTime: null, idleTime: 0, userClass: "", permit: "F" });
		}
		for (var x = 0; x < buddies.length; ++x)			// add buddies
			for (var y = 0; y < buddies[x].offline.length; ++y) {
				this.addBuddy(buddies[x].offline[y].name);
				if (buddies[x].permit == "T")
					this.permitBuddy(buddies[x].offline[y].name);
				else if (buddies[x].permit == "F")
					this.denyBuddy(buddies[x].offline[y].name);
			}
		
		buddyTree.data = buddies;
		buddyTree.treebox.rowCountChanged(0, -buddyTree.rowCount);
		buddyTree.rowCount = buddyTree.data.length;
		buddyTree.treebox.rowCountChanged(0, buddyTree.rowCount);
	},
	addBuddy : function(username) {
		this.sendFlap(this.DATA, "toc_add_buddy " + this.normalize(username));
	},
	removeBuddy : function(username) {
		this.sendFlap(this.DATA, "toc_remove_buddy " + this.normalize(username));
	},
	permitBuddy : function(username) {
		this.sendFlap(this.DATA, "toc_add_permit " + this.normalize(username));
	},
	denyBuddy : function(username) {
		this.sendFlap(this.DATA, "toc_add_deny " + this.normalize(username));
	},
	setConfig : function(configData) {
		this.sendFlap(this.DATA, "toc_set_config " + "\"" + configData + "\"");
	},
	displayInfoUsername : "",
	audioscrobblerCallback : null,
	getInfo : function(username) {
		this.displayInfoUsername = username;
		this.sendFlap(this.DATA, "toc_get_info " + this.normalize(username));
	},
	setInfo : function(info) {
		this.sendFlap(this.DATA, "toc_set_info " + "\"" + this.encode(info) + "\"");
	},
	displayInfo : function(str) {	// XXX this sucks hardcore, have to keep calling until we get a good address!
		var tok = str.split(":");
		var req = new XMLHttpRequest();
		var url = "http://" + this.tocHost + ":" + this.tocPort + "/" + tok[2];
		req.open('GET', url, true);
		req.onreadystatechange = function (aEvt) {
			if (req.readyState == 4 && req.status == 200) {
				mozillaTOC.displayInfoUsername = "";
				if (mozillaTOC.audioscrobblerCallback) {
					var index = req.responseText.indexOf('audioscrobbler.com/user/');
					if (index != -1)
						mozillaTOC.audioscrobblerCallback(req.responseText.substring(index + 24, req.responseText.indexOf(">", index + 24) - 1));
					else
						mozillaTOC.audioscrobblerCallback("<not found>");
					mozillaTOC.audioscrobblerCallback = null;
				} else {
					window.openDialog(url,"infoWindow",
					"screenY="+(document.getElementById('main-window').boxObject.height/2 - 45+document.getElementById('main-window').boxObject.screenY)
				  +",screenX="+(document.getElementById('main-window').boxObject.width/2 - 200+document.getElementById('main-window').boxObject.screenX)+
					",width=400,height=400,chrome=no,modal=no,dialog=no,resizable=yes,scrollbars=yes,toolbars=yes");
				}
			} else if (req.readyState == 4)
				mozillaTOC.sendFlap(mozillaTOC.DATA, "toc_get_info " + mozillaTOC.normalize(mozillaTOC.displayInfoUsername));	// really sucks
		};
		req.send(null);
	},
	handleRvous : function(str) {
		var tok = str.split(":");
		if (tok[1] == this.BUDDY_ICON) {
	//		alert('here');
		}
	},
	send : function(to, msg) {			// Send an IM
		try {
			this.sendFlap(this.DATA, "toc_send_im " + this.normalize(to) + " \"<html><body>" + this.encode(msg) + "</body></html>\"");
		} catch (ex) { }
	},
	setAway : function(away) {
		try {
			this.sendFlap(this.DATA, "toc_set_away " + "\"" + this.encode(away) + "\"");
		} catch (ex) { }
	},
	normalize : function(name) {		// Called to normalize a screen name.  This removes all spaces and converts the name to lower case.
		var rtn = "";
		for (var i = 0; i < name.length; i++) {
			if (name.charAt(i) == ' ')
				continue;
			rtn += name.charAt(i).toLowerCase();
		}

		return rtn;
	},
	encode : function(str) {			// Called to encode a message. Convert carriage returns to <br>'s
		var rtn = "";
		for (var i = 0; i < str.length; i++) {
			switch (str.charAt(i)) {
				case '\n':
					break;
				case '\r':
					rtn += "<br>";
					break;
				case '"':
					rtn += "\\\"";
					break;
				case '{':
				case '}':
				case '\\':
					rtn += "\\";
					// fall through
				default:
					rtn += str.charAt(i);
			}
		}
		rtn = rtn.replace(new RegExp("<br>", "gi"), " ");// get rid of <br>'s
		return rtn;
	},
	handleError : function(str) {			// Parse an error packet
		var tok = tok.split(":");
		var e = tok[1];
		var v = tok.length > 2 ? tok[2] : "";
		var error;
		switch (parseInt(e)) {
			case 901 : error = "$1 not currently available."; break;
			case 902 : error = "Warning of $1 not currently available."; break;
			case 903 : error = "A message has been dropped, you are exceeding the server speed limit."; break;
			case 911 : error = "Error validating input."; break;
			case 912 : error = "Invalid account."; break;
			case 913 : error = "Error encountered while processing request."; break;
			case 914 : error = "Service unavailable."; break;
			case 950 : error = "Chat in $1 is unavailable."; break;
			case 960 : error = "You are sending message too fast to $1."; break;
			case 961 : error = "You missed an im from $1 because it was too big."; break;
			case 962 : error = "You missed an im from $1 because it was sent too fast."; break;
			case 970 : error = "Failure."; break;
			case 971 : error = "Too many matches."; break;
			case 972 : error = "Need more qualifiers."; break;
			case 973 : error = "Dir service temporarily unavailable."; break;
			case 974 : error = "Email lookup restricted."; break;
			case 975 : error = "Keyword ignored."; break;
			case 976 : error = "No Keywords."; break;
			case 977 : error = "Language not supported."; break;
			case 978 : error = "Country not supported."; break;
			case 979 : error = "Failure unknown $1."; break;
			case 980 : error = "Incorrect nickname or password."; break;
			case 981 : error = "The service is temporarily unavailable."; break;
			case 982 : error = "Your warning level is currently too high to sign on."; break;
			case 983 : error = "You have been connecting and disconnecting too frequently.\nWait 10 minutes and try again.\n If you continue to try, you will need to wait even longer."; break;
			case 989 : error = "An unknown signon error has occurred $1."; break;
			default  : error = "Unknown error."; break;
		}
		var i = error.indexOf("$1");
		if (i != -1) {
			error =  error.substring(0, i);
			error += v;
			error += error.substring(i + 2);
		}

		alert(error); // xxx add to im window
	},
	register : function() {
		window.open('https://my.screenname.aol.com/_cqr/login/login.psp?sitedomain=my.screenname.aol.com&authLev=1&siteState=&lang=en&locale=us&createSn=1','Register');
	},
	changePassword : function() {
		window.open('https://aim.aol.com/password/change_password.adp','Change Password');
	},
};
